@@ -156,6 +156,12 @@ class Cov19Model(Model):
             self.sim_shape = (sim_len,)
         elif self.sim_ndim == 2:
             self.sim_shape = (sim_len, self.new_cases_obs.shape[1])
+        elif self.sim_ndim == 3:
+            self.sim_shape = (
+                sim_len,
+                self.new_cases_obs.shape[1],
+                self.new_cases_obs.shape[2],
+            )
 
         if self.data_end > datetime.datetime.today():
             log.warning(
@@ -176,11 +182,11 @@ class Cov19Model(Model):
     @property
     def shape_of_regions(self):
         # Number of regions as tuple of int
-        return () if self.sim_ndim == 1 else self.sim_shape[1]
+        return () if self.sim_ndim == 1 else self.sim_shape[-1]
 
     @property
     def is_hierarchical(self):
-        return self.new_cases_obs.ndim == 2
+        return self.new_cases_obs.ndim >= 2
 
     """
         Forecast properties
